@@ -18,7 +18,7 @@ static const unsigned char sprimes[NSMALLPRIMES] = {2,3,5,7,11,13,17,19,23,29,31
 /* 0 fail, 1 pass, -1 nothing.  Modifies base a */
 static int _preprocess_base(mpz_t n, mpz_t a)
 {
-  if (mpz_cmp_ui(a, 1) < 0)
+  if (mpz_cmp_ui(a, 1) <= 0)
     croak("Base %ld is invalid", mpz_get_si(a));
   if (mpz_cmp_ui(n, 3) <= 0)
     return (mpz_cmp_ui(n, 2) >= 0);
@@ -209,7 +209,7 @@ int is_miller_prime(mpz_t n, int assume_grh)
 
 int miller_rabin_random(mpz_t n, UV numbases, char* seedstr)
 {
-  gmp_randstate_t* p_randstate = get_randstate();
+  gmp_randstate_t randstate;
   mpz_t t, base;
   UV i;
 
@@ -217,19 +217,40 @@ int miller_rabin_random(mpz_t n, UV numbases, char* seedstr)
   if (mpz_cmp_ui(n, 100) < 0)     /* tiny n */
     return (_GMP_is_prob_prime(n) > 0);
 
-  mpz_init(base);  mpz_init(t);
-
-  if (seedstr != 0) { /* Set the RNG seed if they gave us a seed */
-    mpz_set_str(t, seedstr, 0);
-    gmp_randseed(*p_randstate, t);
+  /* See if they've asked for a ludicrous number of bases */
+  mpz_init(t);
+  mpz_mul_ui(t, n, 3);
+  mpz_div_ui(t, t, 4);
+  if (mpz_cmp_ui(t, numbases) <= 0) {
+    int res = is_bpsw_dmr_prime(n);
+    if (res != 1) {
+      mpz_clear(t);
+      return !!res;
+    }
+    numbases = mpz_get_ui(t);
   }
 
+  mpz_init(base);
   mpz_sub_ui(t, n, 3);
-  for (i = 0; i < numbases; i++) {
-    mpz_urandomm(base, *p_randstate, t);  /* base = 0 .. (n-3)-1 */
-    mpz_add_ui(base, base, 2);            /* base = 2 .. n-2     */
-    if (miller_rabin(n, base) == 0)
-      break;
+
+  if (seedstr == 0) { /* Normal case with no seed string.  Use our CSPRNG. */
+    for (i = 0; i < numbases; i++) {
+      mpz_isaac_urandomm(base, t);    /* base 0 .. (n-3)-1 */
+      mpz_add_ui(base, base, 2);      /* base 2 .. n-2     */
+      if (miller_rabin(n, base) == 0)
+        break;
+    }
+  } else {
+    gmp_randinit_default(randstate);
+    mpz_set_str(base, seedstr, 0);
+    gmp_randseed(randstate, base);
+    for (i = 0; i < numbases; i++) {
+      mpz_urandomm(base, randstate, t); /* base 0 .. (n-3)-1 */
+      mpz_add_ui(base, base, 2);        /* base 2 .. n-2     */
+      if (miller_rabin(n, base) == 0)
+        break;
+    }
+    gmp_randclear(randstate);
   }
   mpz_clear(base);  mpz_clear(t);
   return (i >= numbases);
@@ -1104,7 +1125,6 @@ int is_frobenius_pseudoprime(mpz_t n, IV P, IV Q)
 /* Use Crandall/Pomerance, steps from Loebenberger 2008 */
 int is_frobenius_cp_pseudoprime(mpz_t n, UV ntests)
 {
-  gmp_randstate_t* p_randstate = get_randstate();
   mpz_t t, a, b, d, w1, wm, wm1, m;
   UV tn;
   int j;
@@ -1121,9 +1141,9 @@ int is_frobenius_cp_pseudoprime(mpz_t n, UV ntests)
     /* Step 1: choose a and b in 1..n-1 and d=a^2-4b not square and coprime */
     do {
       mpz_sub_ui(t, n, 1);
-      mpz_urandomm(a, *p_randstate, t);
+      mpz_isaac_urandomm(a, t);
       mpz_add_ui(a, a, 1);
-      mpz_urandomm(b, *p_randstate, t);
+      mpz_isaac_urandomm(b, t);
       mpz_add_ui(b, b, 1);
       /* Check d and gcd */
       mpz_mul(d, a, a);
@@ -1261,7 +1281,7 @@ int _GMP_is_frobenius_underwood_pseudoprime(mpz_t n)
   mpz_mod(temp1, temp1, n);
   if (mpz_cmp_ui(s, 0) == 0 && mpz_cmp(t, temp1) == 0)
     rval = 1;
-  if (_verbose>1) gmp_printf("%Zd is %s with a = %"UVuf"\n", n, (rval) ? "probably prime" : "composite", a);
+  if (_verbose>1) { gmp_printf("%Zd is %s with a = %"UVuf"\n", n, (rval) ? "probably prime" : "composite", a); fflush(stdout); }
 
   mpz_clear(temp1); mpz_clear(temp2); mpz_clear(n_plus_1);
   mpz_clear(s); mpz_clear(t);
@@ -1342,7 +1362,7 @@ int _GMP_is_frobenius_khashin_pseudoprime(mpz_t n)
 int _GMP_BPSW(mpz_t n)
 {
   if (mpz_cmp_ui(n, 4) < 0)
-    return (mpz_cmp_ui(n, 1) <= 0) ? 0 : 1;
+    return (mpz_cmp_ui(n, 1) <= 0) ? 0 : 2;
 
   if (miller_rabin_ui(n, 2) == 0)   /* Miller Rabin with base 2 */
     return 0;
